// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KGANTT_VERSION_H
#define KGANTT_VERSION_H

#define KGANTT_VERSION_STRING "3.0.1"
#define KGANTT_VERSION_MAJOR 3
#define KGANTT_VERSION_MINOR 0
#define KGANTT_VERSION_PATCH 1
#define KGANTT_VERSION ((3<<16)|(0<<8)|(1))

#endif
